@@ -1,19 +1,9 @@
-"""
-This module contains the `mkdocs` plugin.
+"""This module contains the "mkdocstrings" plugin for MkDocs.
 
 The plugin instantiates a Markdown extension ([`MkdocstringsExtension`][mkdocstrings.extension.MkdocstringsExtension]),
 and adds it to the list of Markdown extensions used by `mkdocs`
 during the [`on_config` event hook](https://www.mkdocs.org/user-guide/plugins/#on_config).
 
-After each page is processed by the Markdown converter, this plugin stores absolute URLs of every HTML anchors
-it finds to later be able to fix unresolved references.
-It stores them during the [`on_page_contents` event hook](https://www.mkdocs.org/user-guide/plugins/#on_page_contents).
-
-Just before writing the final HTML to the disc, during the
-[`on_post_page` event hook](https://www.mkdocs.org/user-guide/plugins/#on_post_page),
-this plugin searches for references of the form `[identifier][]` or `[title][identifier]` that were not resolved,
-and fixes them using the previously stored identifier-URL mapping.
-
 Once the documentation is built, the [`on_post_build` event hook](https://www.mkdocs.org/user-guide/plugins/#on_post_build)
 is triggered and calls the [`handlers.teardown()` method][mkdocstrings.handlers.base.Handlers.teardown]. This method is
 used to teardown the handlers that were instantiated during documentation buildup.
@@ -22,21 +12,19 @@ Finally, when serving the documentation, it can add directories to watch
 during the [`on_serve` event hook](https://www.mkdocs.org/user-guide/plugins/#on_serve).
 """
 
-import logging
 import os
-from typing import Any, Callable, Dict, Optional, Tuple
+from typing import Callable, Optional, Tuple
 
 from livereload import Server
 from mkdocs.config import Config
 from mkdocs.config.config_options import Type as MkType
 from mkdocs.plugins import BasePlugin
-from mkdocs.structure.pages import Page
-from mkdocs.structure.toc import AnchorLink
+from mkdocs.utils import write_file
+from mkdocs_autorefs.plugin import AutorefsPlugin
 
 from mkdocstrings.extension import MkdocstringsExtension
 from mkdocstrings.handlers.base import BaseHandler, Handlers
 from mkdocstrings.loggers import get_logger
-from mkdocstrings.references import fix_refs
 
 log = get_logger(__name__)
 
@@ -47,19 +35,16 @@ RENDERING_OPTS_KEY: str = "rendering"
 
 
 class MkdocstringsPlugin(BasePlugin):
-    """
-    An `mkdocs` plugin.
+    """An `mkdocs` plugin.
 
     This plugin defines the following event hooks:
 
     - `on_config`
-    - `on_page_contents`
-    - `on_post_page`
     - `on_post_build`
     - `on_serve`
 
     Check the [Developing Plugins](https://www.mkdocs.org/user-guide/plugins/#developing-plugins) page of `mkdocs`
-    for more information about its plugin system..
+    for more information about its plugin system.
     """
 
     config_scheme: Tuple[Tuple[str, MkType]] = (
@@ -99,15 +84,29 @@ class MkdocstringsPlugin(BasePlugin):
     ```
     """
 
+    css_filename = "assets/_mkdocstrings.css"
+
     def __init__(self) -> None:
         """Initialize the object."""
         super().__init__()
-        self.url_map: Dict[Any, str] = {}
-        self.handlers: Optional[Handlers] = None
+        self._handlers: Optional[Handlers] = None
+
+    @property
+    def handlers(self) -> Handlers:
+        """Get the instance of [mkdocstrings.handlers.base.Handlers][] for this plugin/build.
+
+        Raises:
+            RuntimeError: If the plugin hasn't been initialized with a config.
+
+        Returns:
+            An instance of [mkdocstrings.handlers.base.Handlers][] (the same throughout the build).
+        """
+        if not self._handlers:
+            raise RuntimeError("The plugin hasn't been initialized with a config yet")
+        return self._handlers
 
     def on_serve(self, server: Server, builder: Callable = None, **kwargs) -> Server:  # noqa: W0613 (unused arguments)
-        """
-        Watch directories.
+        """Watch directories.
 
         Hook for the [`on_serve` event](https://www.mkdocs.org/user-guide/plugins/#on_serve).
         In this hook, we add the directories specified in the plugin's configuration to the list of directories
@@ -132,8 +131,7 @@ class MkdocstringsPlugin(BasePlugin):
         return server
 
     def on_config(self, config: Config, **kwargs) -> Config:  # noqa: W0613 (unused arguments)
-        """
-        Instantiate our Markdown extension.
+        """Instantiate our Markdown extension.
 
         Hook for the [`on_config` event](https://www.mkdocs.org/user-guide/plugins/#on_config).
         In this hook, we instantiate our [`MkdocstringsExtension`][mkdocstrings.extension.MkdocstringsExtension]
@@ -163,85 +161,30 @@ class MkdocstringsPlugin(BasePlugin):
             "mdx_configs": config["mdx_configs"],
             "mkdocstrings": self.config,
         }
+        self._handlers = Handlers(extension_config)
 
-        self.handlers = Handlers(extension_config)
-        mkdocstrings_extension = MkdocstringsExtension(extension_config, self.handlers)
+        try:
+            # If autorefs plugin is explicitly enabled, just use it.
+            autorefs = config["plugins"]["autorefs"]
+            log.debug(f"Picked up existing autorefs instance {autorefs!r}")
+        except KeyError:
+            # Otherwise, add a limited instance of it that acts only on what's added through `register_anchor`.
+            autorefs = AutorefsPlugin()
+            autorefs.scan_toc = False
+            config["plugins"]["autorefs"] = autorefs
+            log.debug(f"Added a subdued autorefs instance {autorefs!r}")
+        # Add collector-based fallback in either case.
+        autorefs.get_fallback_anchor = self._handlers.get_anchor
+
+        mkdocstrings_extension = MkdocstringsExtension(extension_config, self._handlers, autorefs)
         config["markdown_extensions"].append(mkdocstrings_extension)
+
+        config["extra_css"].insert(0, self.css_filename)  # So that it has lower priority than user files.
+
         return config
 
-    def on_page_content(self, html: str, page: Page, **kwargs) -> str:  # noqa: W0613 (unused arguments)
-        """
-        Map anchors to URLs.
-
-        Hook for the [`on_page_contents` event](https://www.mkdocs.org/user-guide/plugins/#on_page_contents).
-        In this hook, we map the IDs of every anchor found in the table of contents to the anchors absolute URLs.
-        This mapping will be used later to fix unresolved reference of the form `[title][identifier]` or
-        `[identifier][]`.
-
-        Arguments:
-            html: HTML converted from Markdown.
-            page: The related MkDocs page instance.
-            kwargs: Additional arguments passed by MkDocs.
-
-        Returns:
-            The same HTML. We only use this hook to map anchors to URLs.
-        """
-        log.debug(f"Mapping identifiers to URLs for page {page.file.src_path}")
-        for item in page.toc.items:
-            self.map_urls(page.url, item)
-        return html
-
-    def map_urls(self, base_url: str, anchor: AnchorLink) -> None:
-        """
-        Recurse on every anchor to map its ID to its absolute URL.
-
-        This method populates `self.url_map` by side-effect.
-
-        Arguments:
-            base_url: The base URL to use as a prefix for each anchor's relative URL.
-            anchor: The anchor to process and to recurse on.
-        """
-        self.url_map[anchor.id] = base_url + anchor.url
-        for child in anchor.children:
-            self.map_urls(base_url, child)
-
-    def on_post_page(self, output: str, page: Page, **kwargs) -> str:  # noqa: W0613 (unused arguments)
-        """
-        Fix cross-references.
-
-        Hook for the [`on_post_page` event](https://www.mkdocs.org/user-guide/plugins/#on_post_page).
-        In this hook, we try to fix unresolved references of the form `[title][identifier]` or `[identifier][]`.
-        Doing that allows the user of `mkdocstrings` to cross-reference objects in their documentation strings.
-        It uses the native Markdown syntax so it's easy to remember and use.
-
-        We log a warning for each reference that we couldn't map to an URL, but try to be smart and ignore identifiers
-        that do not look legitimate (sometimes documentation can contain strings matching
-        our [`AUTO_REF_RE`][mkdocstrings.references.AUTO_REF_RE] regular expression that did not intend to reference anything).
-        We currently ignore references when their identifier contains a space or a slash.
-
-        Arguments:
-            output: HTML converted from Markdown.
-            page: The related MkDocs page instance.
-            kwargs: Additional arguments passed by MkDocs.
-
-        Returns:
-            Modified HTML.
-        """
-        log.debug(f"Fixing references in page {page.file.src_path}")
-
-        fixed_output, unmapped = fix_refs(output, page.url, self.url_map)
-
-        if unmapped and log.isEnabledFor(logging.WARNING):
-            for ref in unmapped:
-                log.warning(
-                    f"{page.file.src_path}: Could not find cross-reference target '[{ref}]'",
-                )
-
-        return fixed_output
-
-    def on_post_build(self, **kwargs) -> None:  # noqa: W0613,R0201 (unused arguments, cannot be static)
-        """
-        Teardown the handlers.
+    def on_post_build(self, config: Config, **kwargs) -> None:  # noqa: W0613,R0201 (unused arguments, cannot be static)
+        """Teardown the handlers.
 
         Hook for the [`on_post_build` event](https://www.mkdocs.org/user-guide/plugins/#on_post_build).
         This hook is used to teardown all the handlers that were instantiated and cached during documentation buildup.
@@ -253,25 +196,23 @@ class MkdocstringsPlugin(BasePlugin):
         this hook.
 
         Arguments:
+            config: The MkDocs config object.
             kwargs: Additional arguments passed by MkDocs.
         """
-        if self.handlers:
+        if self._handlers:
+            css_content = "\n".join(handler.renderer.extra_css for handler in self.handlers.seen_handlers)
+            write_file(css_content.encode("utf-8"), os.path.join(config["site_dir"], self.css_filename))
+
             log.debug("Tearing handlers down")
-            self.handlers.teardown()
+            self._handlers.teardown()
 
     def get_handler(self, handler_name: str) -> BaseHandler:
-        """
-        Get a handler by its name. See [mkdocstrings.handlers.base.Handlers.get_handler][].
+        """Get a handler by its name. See [mkdocstrings.handlers.base.Handlers.get_handler][].
 
         Arguments:
             handler_name: The name of the handler.
 
-        Raises:
-            RuntimeError: If the plugin hasn't been initialized with a config.
-
         Returns:
             An instance of a subclass of [`BaseHandler`][mkdocstrings.handlers.base.BaseHandler].
         """
-        if not self.handlers:
-            raise RuntimeError("The plugin hasn't been initialized with a config yet")
         return self.handlers.get_handler(handler_name)
